import 'dart:async';

import 'package:flutter/services.dart';

typedef _AutoUssdSessionExecutionCallback = void Function(AutoUssdResult result);

class AutoUssdFlutter {
  static const String _CHANNEL_NAME = "autoussdflutter";
  static const String _CALLBACK_METHOD_NAME = "onSessionCallback";
  static const MethodChannel _CHANNEL = const MethodChannel(
    _CHANNEL_NAME,
  );

  final _AutoUssdSessionExecutionCallback _onSessionExecuted;

  /// Primary constructor of the library.
  ///
  /// USSD session execution is asynchronous on the native platform
  /// thus a callback must be specified to be notified of the result
  AutoUssdFlutter(this._onSessionExecuted) {
    _CHANNEL.setMethodCallHandler((call) async {
      if (call.method == _CALLBACK_METHOD_NAME) {
        final List<dynamic> args = call.arguments;
        final sessionId = args[0] as String;
        final status = args[1] as String;
        final description = args[2] as String;
        final lastContent = args[3] as String?;
        final parser = args[4] as String?;

        _onSessionExecuted(
          AutoUssdResult._(
            sessionId,
            status,
            description,
            lastContent,
            parser,
          ),
        );
      }
    });
  }

  /// Executes a session matching the specified in your app.
  ///
  /// Visit the [docs](https://medium.com/@autoussd/sessions-48607f1dc639)
  /// for more information on sessions and how to configure them in your
  /// console
  Future<void> executeSession(
    String sessionId, [
    List<String>? variables,
  ]) async {
    final args = {
      "sessionId": sessionId,
      "variables": variables,
    };

    await _CHANNEL.invokeMethod(
      "executeSession",
      args,
    );
  }
}

/// Represents the result of a session execution
///
/// After the SDK executes a session it asynchronously returns an object
/// of this type which contains information on what went on during the execution
class AutoUssdResult {
  final String sessionId;
  final AutoUssdResultStatus status;
  final String description;
  final String? lastContent;
  final String? parser;

  AutoUssdResult._(
    this.sessionId,
    String status,
    this.description,
    this.lastContent,
    this.parser,
  ) : this.status = AutoUssdResult._parseStatus(status);

  static AutoUssdResultStatus _parseStatus(String status) {
    switch (status) {
      case "COMPLETED":
        return AutoUssdResultStatus.COMPLETED;
      case "PARSED":
        return AutoUssdResultStatus.PARSED;
      case "INVALID_SESSION":
        return AutoUssdResultStatus.INVALID_SESSION;
      case "SESSION_TIMEOUT":
        return AutoUssdResultStatus.SESSION_TIMEOUT;
      case "MENU_CONTENT_MISMATCH":
        return AutoUssdResultStatus.MENU_CONTENT_MISMATCH;
      case "ACCOUNT_SUBSCRIPTION_EXPIRED":
        return AutoUssdResultStatus.ACCOUNT_SUBSCRIPTION_EXPIRED;
      case "UNKNOWN_ERROR":
        return AutoUssdResultStatus.UNKNOWN_ERROR;
      default:
        throw "Unknown status: $status";
    }
  }
}

/// Enum constants for report statuses */
enum AutoUssdResultStatus {
  /// Indicates that a session (without SMS parsers) executed successfully */
  COMPLETED,

  /// Indicates that a session execution pending SMS confirmation has been parsed */
  PARSED,

  /// Indicates that the session does not exist in app's sessions */
  INVALID_SESSION,

  /// Indicates that the session timed-out during execution */
  SESSION_TIMEOUT,

  /// Indicates that the menu match text didn't match that of the USSD dialog content */
  MENU_CONTENT_MISMATCH,

  /// Indicates that the session owner's subscription period has elapsed */
  ACCOUNT_SUBSCRIPTION_EXPIRED,

  /// Indicates that an unknown error occurred */
  UNKNOWN_ERROR,
}
